#!/bin/bash -euo pipefail
# Rename gff and remove sequence entries
# bakta has region entries
awk '$3 == "CDS"' prokka_gff | grep "ID=" > vibrio31.gff ;

## Increment GFF with custom annotations
### VFDB
if [ ! $(cat vibrio31_vfdb_blastn_onGenes.txt | wc -l) -le 1 ]
then
  addBlast2Gff.R -i vibrio31_vfdb_blastn_onGenes.txt -g vibrio31.gff -o vibrio31.gff -d VFDB -t Virulence ;
  grep "VFDB" vibrio31.gff > virulence_vfdb.gff ;
fi

### Victors
if [ ! $(cat vibrio31_victors_blastp_onGenes.txt | wc -l) -le 1 ]
then 
  addBlast2Gff.R -i vibrio31_victors_blastp_onGenes.txt -g vibrio31.gff -o vibrio31.gff -d Victors -t Virulence ;
  grep "Victors" vibrio31.gff > virulence_victors.gff ;
fi

### KEGG Orthology
## Reformat KOfamscan Output
if [ ! $(cat vibrio31_ko_forKEGGMapper.txt | wc -l) -eq 0 ]
then
  awk \
    -F'\t' \
    -v OFS='\t' \
    '{x=$1;$1="";a[x]=a[x]$0}END{for(x in a)print x,a[x]}' \
    vibrio31_ko_forKEGGMapper.txt  | \
  sed \
    -e 's/\t/,/g' \
    -e 's/,,/\t/g' | \
  awk  '$2!=""' > formated.txt ;
  addKO2Gff.R -i formated.txt -g vibrio31.gff -o vibrio31.gff -d KEGG ;
fi

### ICEs
if [ ! $(cat vibrio31_iceberg_blastp_onGenes.txt | wc -l) -le 1 ]
then
  addBlast2Gff.R -i vibrio31_iceberg_blastp_onGenes.txt -g vibrio31.gff -o vibrio31.gff -d ICEberg -t ICE ;
  grep "ICEberg" vibrio31.gff > ices_iceberg.gff ;
fi

### Prophages
if [ ! $(cat vibrio31_phast_blastp_onGenes.txt | wc -l) -le 1 ]
then
  addBlast2Gff.R -i vibrio31_phast_blastp_onGenes.txt -g vibrio31.gff -o vibrio31.gff -d PHAST -t Prophage ;
  grep "PHAST" vibrio31.gff > prophages_phast.gff ;
fi

### Resistance
#### RGI
if [ ! $(cat RGI_vibrio31.txt | wc -l) -le 1 ]
then
  addRGI2gff.R -g vibrio31.gff -i RGI_vibrio31.txt -o vibrio31.gff ;
  grep "CARD" vibrio31.gff > resistance_card.gff ;
fi

#### AMRFinderPlus
if [ ! $(cat AMRFinder_resistance-only.tsv | wc -l) -le 1 ]
then 
  addNCBIamr2Gff.R -g vibrio31.gff -i AMRFinder_resistance-only.tsv -o vibrio31.gff -t Resistance -d AMRFinderPlus ;
  grep "AMRFinderPlus" vibrio31.gff > resistance_amrfinderplus.gff ;
fi

#### Resfinder
if [ ! $(cat results_tab.gff | wc -l) -eq 0 ]
then
  bedtools intersect -a results_tab.gff -b vibrio31.gff -wo | sort -k19,19 -r | awk -F '\t' '!seen[$9]++' > resfinder_intersected.txt ;
  addBedtoolsIntersect.R -g vibrio31.gff -t resfinder_intersected.txt --type Resistance --source Resfinder -o vibrio31.gff ;
  grep "Resfinder" vibrio31.gff > resistance_resfinder.gff ;
  rm -f resfinder_intersected.txt ;
fi

#### Custom Blast databases
for file in input.11 ;
do
  if [ ! $(cat $file | wc -l) -eq 0 ]
  then
    db=${file%%_custom_db.gff} ;
    bedtools intersect -a ${file} -b vibrio31.gff -wo | sort -k19,19 -r | awk -F '\t' '!seen[$9]++' > bedtools_intersected.txt ;
    addBedtoolsIntersect.R -g vibrio31.gff -t bedtools_intersected.txt --type "CDS" --source "${db}" -o vibrio31.gff ;
    grep "${db}" vibrio31.gff > custom_database_${db}.gff ;
    rm -f bedtools_intersected.txt ;
  fi
done

### digIS transposable elements
touch transposable_elements_digis.gff
if [ -s digis_gff ]
then
  ( cat digis_gff | sed 's/id=/ID=/g' > transposable_elements_digis.gff && rm digis_gff ) ;
  cat vibrio31.gff transposable_elements_digis.gff | bedtools sort > tmp.out.gff ;
  ( cat tmp.out.gff > vibrio31.gff && rm tmp.out.gff );
fi

### integron_finder results
### integrons are unique / complete elements and should not be intersected
cat vibrio31.gff vibrio31_integrons.gff | bedtools sort > tmp.gff ;
cat tmp.gff > vibrio31.gff
rm tmp.gff
